
import os.path
import urllib2
import re
import BeautifulSoup
import codecs
import crawler_package.utils, crawler_package.html_checker, crawler_package.clean_up_text_file


elementsx = [ ]
text_file_extension = ".txt"

def convert_bad_text_file(filename):
    content = open(filename,'r').readlines()
    (directory, fn) = os.path.split(filename)
    write_content_file(directory,filename,content, clean=True)

def write_content_file(write_dir, filename, content, clean=False,
                       text_file_extension=text_file_extension):
    if clean:
        new_content = crawler_package.html_checker.clean_up_lines(content)
    else:
        new_content = content
    crawler_package.utils.make_dir_if_necessary(write_dir)
    (base,old_extension) = os.path.splitext(filename)
    full_path = os.path.join(write_dir,  base+text_file_extension)
    try:
        fh = open(full_path, 'w')
    except IOError, message:
        ## Errors may be getting trapped above
        ## This info important
        print 'write_content_file IOError', message
        raise IOError, message
    ## \n' join better than ' ': e.g., separate XML doc info lines from content
    new_contents = '\n'.join(new_content)
    print >> fh, codecs.utf_8_encode(new_contents,'ignore')[0]
    fh.close()
    (full_path0, ext) = os.path.splitext(full_path)
    ## Return to allow auxiliary info to be stored in other files.
    return full_path0

def write_website_file(write_dir, url, content, clean=True):
    if clean:
        content = crawler_package.html_checker.clean_up_lines(content)
    ## data_dir is named after website. Preserve website structure
    (data_dir, path_list) = url2filename(url)
    ## Separating what will be absolute directory path from file name.
    dir_path = os.path.join(write_dir, data_dir, *path_list[:-1])
    crawler_package.utils.make_dir_if_necessary(dir_path)
    full_path = os.path.join(dir_path,path_list[-1])
    print full_path
    try:
        fh = open(full_path, 'w')
    except IOError, message:
        ## Errors getting trapped above by crawler.
        ## This info important
        print 'IOError', message
    print >> fh, codecs.utf_8_encode('\n'.join(content),'ignore')[0]
    fh.close()
    (full_path0, ext) = os.path.splitext(full_path)
    ## Return to allow auxiliary info to be stored in other files.
    return full_path0

url_sep_pattern = re.compile('[:\/]')
elem_separator = '+'

def url2filename (url):
    urlparser = urllib2.urlparse.urlparse
    (scheme, netloc, path, params, query, fragment) = urlparser(url)
    suffixes = [ ]
    for s in [params, query, fragment]:
        if s:
            if suffixes:
                suffixes.append(elem_separator+s)
            else:
                suffixes.append(s)
    # Do this for op system portability
    (path0, ext) = os.path.splitext(path)
    path1 = path0 + ''.join(suffixes) + os.path.extsep + 'txt'
    p_path = [s for s in url_sep_pattern.split(path1) if s]
    ## Strip default server name
    netloc = netloc.lstrip('www.')
    return (netloc, p_path)


def new_collect_text_lines(accumulator, elements, omit_comments=True):
    # elements.append(accumulator)
    print 'collect_text_lines'
    collect_text_lines_helper(accumulator, elements, omit_comments=True)
    return accumulator

def collect_text_lines(accumulator, elements, omit_comments=True):
    elements.extend(accumulator)
    print 'collect_text_lines'
    collect_text_lines_helper(accumulator, elements, omit_comments=True)
    return accumulator

def collect_text_lines_helper(accumulator, elements, omit_comments=True):
    omit_tags = ['script']
    for x in elements:
         try:
             # Only tags have names and contents
             if x.name not in omit_tags:
                 partsList = x.contents
             else:
                 partsList = [ ]
         except AttributeError:
             # No name or contents attr means this is a navigable string
             # A navigable string returned by contents has no subparts.
             # recursion bottoms out
             partsList = [ ]
             if omit_comments and x.__class__ is BeautifulSoup.Comment:
                 ## Comments are a subclass of NavigableString
                 ## Leave them out.
                 pass
             else:
                 accumulator.append(decodeHTML(x))
         collect_text_lines_helper(accumulator, partsList,omit_comments)


def old_convert_html_file (full_path, write_dir=None, \
                       text_file_extension=text_file_extension,\
                       verbose=False):
    if verbose:
        print 'Converting %s' % full_path
    fh = open(full_path,'r')
    soup = BeautifulSoup.BeautifulSoup()
    soup.feed(unicode(''.join(fh.readlines()), errors="ignore"))
    content = collect_text_lines([],[soup])
    (html_directory, filename) = os.path.split(full_path)
    if write_dir is None:
        write_dir = html_directory
    return write_content_file(write_dir,filename, content, \
                              text_file_extension=text_file_extension)

def convert_html_file (full_path, write_dir=None, \
                       text_file_extension=text_file_extension,\
                       verbose=False,new_base=None):
    if verbose:
        print 'Converting %s' % full_path
    fh = open(full_path,'r')
    soup = BeautifulSoup.BeautifulSoup()
    soup.feed(unicode(''.join(fh.readlines()), errors="ignore"))
    content = new_collect_text_lines([],[soup])
    (html_directory, filename) = os.path.split(full_path)
    if write_dir is None:
        write_dir = html_directory
    if new_base:
        (old_base,ext) = os.path.splitext(filename)
        filename = new_base + ext
    return write_content_file(write_dir,filename, content, \
                              text_file_extension=text_file_extension)


def decodeHTML(s=""):
    """encodeHTML(s) -> str

    Decode HTML from HTML entities to ASCII form.
    """

    return s.replace( "&amp;", "&") \
           .replace("&#34;",  """ \"  """) \
           .replace("&#38;",   """ &  """) \
           .replace("&#60;",   """ <  """) \
           .replace("&#62;",   """ >  """) \
           .replace("&#160;",   """    """) \
           .replace("&#161;",   """ !  """) \
           .replace("&#162;",   """ cents  """) \
           .replace("&#163;",   """  pounds """) \
           .replace("&#164;",   """     """) \
           .replace("&#165;",   """  yen  """) \
           .replace("&#166;",   """  \mid  """) \
           .replace("&#167;",   """  Section  """) \
           .replace("&#168;",   """    """) \
           .replace("&#169;",   """  copyright  """) \
           .replace("&#170;",   """    """) \
           .replace("&#171;",   """    """) \
           .replace("&#172;",   """   not """) \
           .replace("&#173;",   """   --  """) \
           .replace("&#174;",   """ trademark  """) \
           .replace("&#175;",   """    """) \
           .replace("&#176;",   """  degrees  """) \
           .replace("&#177;",   """  plus or minus """) \
           .replace("&#178;",   """  ^2  """) \
           .replace("&#179;",   """  ^3  """) \
           .replace("&#180;",   """    """) \
           .replace("&#181;",   """    """) \
           .replace("&#182;",   """    """) \
           .replace("&#183;",   """   .   """) \
           .replace("&#184;",   """     """) \
           .replace("&#185;",   """   ^1  """) \
           .replace("&#186;",   """     """) \
           .replace("&#187;",   """     """) \
           .replace("&#188;",   """   1/4  """) \
           .replace("&#189;",   """   1/2  """) \
           .replace("&#190;",   """   3/4  """) \
           .replace("&#191;",   """  ?   """) \
           .replace("&#192;",   """A""") \
           .replace("&#193;",   """A""") \
           .replace("&#194;",   """A""") \
           .replace("&#195;",   """A""") \
           .replace("&#196;",   """A""") \
           .replace("&#197;",   """A""") \
           .replace("&#198;",   """AE""") \
           .replace("&#199;",   """C""") \
           .replace("&#200;",   """E""") \
           .replace("&#201;",   """E""") \
           .replace("&#202;",   """E""") \
           .replace("&#203;",   """E""") \
           .replace("&#204;",   """I""") \
           .replace("&#205;",   """I""") \
           .replace("&#206;",   """I""") \
           .replace("&#207;",   """I""") \
           .replace("&#208;",   """TH""") \
           .replace("&#209;",   """N""") \
           .replace("&#210;",   """O""") \
           .replace("&#211;",   """O""") \
           .replace("&#212;",   """O""") \
           .replace("&#213;",   """O""") \
           .replace("&#214;",   """O""") \
           .replace("&#215;",   """  X   """) \
           .replace("&#216;",   """O""") \
           .replace("&#217;",   """U""") \
           .replace("&#218;",   """U""") \
           .replace("&#219;",   """U""") \
           .replace("&#220;",   """U""") \
           .replace("&#221;",   """Y""") \
           .replace("&#222;",   """TH""") \
           .replace("&#223;",   """sz""") \
           .replace("&#224;",   """a""") \
           .replace("&#225;",   """a""") \
           .replace("&#226;",   """a""") \
           .replace("&#227;",   """a""") \
           .replace("&#228;",   """a""") \
           .replace("&#229;",   """a""") \
           .replace("&#230;",   """ae """) \
           .replace("&#231;",   """c""") \
           .replace("&#232;",   """e""") \
           .replace("&#233;",   """e""") \
           .replace("&#234;",   """e""") \
           .replace("&#235;",   """e""") \
           .replace("&#236;",   """i""") \
           .replace("&#237;",   """i""") \
           .replace("&#238;",   """i""") \
           .replace("&#239;",   """i""") \
           .replace("&#240;",   """th""") \
           .replace("&#241;",   """n""") \
           .replace("&#242;",   """o""") \
           .replace("&#243;",   """o""") \
           .replace("&#244;",   """o""") \
           .replace("&#245;",   """o""") \
           .replace("&#246;",   """o""") \
           .replace("&#247;",   """/""") \
           .replace("&#248;",   """o""") \
           .replace("&#249;",   """u""") \
           .replace("&#250;",   """u""") \
           .replace("&#251;",   """u""") \
           .replace("&#252;",   """u""") \
           .replace("&#253;",   """y""") \
           .replace("&#254;",   """th""") \
           .replace("&#255;",   """y""") \
           .replace("&#338;",   """OE""") \
           .replace("&#339;",   """oe""") \
           .replace("&#ligature;",   """   """) \
           .replace("&#352;",   """S""") \
           .replace("&#353;",   """S""") \
           .replace("&#376;",   """Y""") \
           .replace("&#402;",   """f""") \
           .replace("&#710;",   "") \
           .replace("&#732;",   "") \
           .replace("&#913;",   """  	Alpha """) \
           .replace("&#914;",   """  	Beta """) \
           .replace("&#915;",   """  	Gamma """) \
           .replace("&#916;",   """  	Delta """) \
           .replace("&#917;",   """  	Epsilon """) \
           .replace("&#918;",   """  	Zeta """) \
           .replace("&#919;",   """  	Eta """) \
           .replace("&#920;",   """  	Theta """) \
           .replace("&#921;",   """  	Iota """) \
           .replace("&#922;",   """  	Kappa """) \
           .replace("&#923;",   """  	Lambda """) \
           .replace("&#924;",   """  	Mu """) \
           .replace("&#925;",   """  	Nu """) \
           .replace("&#926;",   """  	Xi """) \
           .replace("&#927;",   """  	Omicron """) \
           .replace("&#928;",   """  	Pi """) \
           .replace("&#929;",   """  	Rho """) \
           .replace("&#there;",   """   is """) \
           .replace("&#931;",   """  	Sigma """) \
           .replace("&#932;",   """  	Tau """) \
           .replace("&#933;",   """  	Upsilon """) \
           .replace("&#934;",   """  	Phi """) \
           .replace("&#935;",   """  	Chi """) \
           .replace("&#936;",   """  	Psi """) \
           .replace("&#937;",   """  	Omega """) \
           .replace("&#945;",   """  	alpha """) \
           .replace("&#946;",   """  	beta """) \
           .replace("&#947;",   """  	gamma """) \
           .replace("&#948;",   """  	delta """) \
           .replace("&#949;",   """  	epsilon """) \
           .replace("&#950;",   """  	zeta """) \
           .replace("&#951;",   """  	eta """) \
           .replace("&#952;",   """  	theta """) \
           .replace("&#953;",   """  	iota """) \
           .replace("&#954;",   """  	kappa """) \
           .replace("&#955;",   """  	lambda """) \
           .replace("&#956;",   """  	mu """) \
           .replace("&#957;",   """  	nu """) \
           .replace("&#958;",   """  	xi """) \
           .replace("&#959;",   """  	omicron """) \
           .replace("&#960;",   """  	pi """) \
           .replace("&#961;",   """  	rho """) \
           .replace("&#962;",   """  	sigmaf """) \
           .replace("&#963;",   """  	sigma """) \
           .replace("&#964;",   """  	tau """) \
           .replace("&#965;",   """  	upsilon """) \
           .replace("&#966;",   """  	phi """) \
           .replace("&#967;",   """  	chi """) \
           .replace("&#968;",   """  	psi """) \
           .replace("&#969;",   """  	omega """) \
           .replace("&#977;",   """  	theta """) \
           .replace("&#978;",   """  	upsih """) \
           .replace("&#982;",   """  	pi """) \
           .replace("&#8194;",   """   """) \
           .replace("&#8195;",   """   """) \
           .replace("&#8201;",   """   """) \
           .replace("&#8204;",   """   """) \
           .replace("&#8205;",   """   """) \
           .replace("&#8206;",   """   """) \
           .replace("&#8207;",   """   """) \
           .replace("&#8211;",   """  -  """) \
           .replace("&#8212;",   """  --  """) \
           .replace("&#8216;",   """ '  """) \
           .replace("&#8217;",   """  ' """) \
           .replace("&#8218;",   """  '   """) \
           .replace("&#8220;",   """  \"   """) \
           .replace("&#8221;",   """  \"  """) \
           .replace("&#8222;",   """  \"  """) \
           .replace("&#8224;",   """  --  """) \
           .replace("&#8225;",   """  --  """) \
           .replace("&#8226;",   """  --  """) \
           .replace("&#bullet;",   """  --  """) \
           .replace("&#8230;",   """  ...  """) \
           .replace("&#8240;",   """  per million """) \
           .replace("&#8242;",   """'   """) \
           .replace("&#8243;",   """'   """) \
           .replace("&#8249;",   """  '   """) \
           .replace("&#lsaquo;",   """  ' """) \
           .replace("&#8250;",   """  '  """) \
           .replace("&#rsaquo;",   """  '  """) \
           .replace("&#8254;",   """     """) \
           .replace("&#8260;",   """ /  """) \
           .replace("&#8364;",   """   """) \
           .replace("&#8465;",   """  euros  """) \
           .replace("&#8472;",   """  powerset """) \
           .replace("&#8476;",   """  Real Numbers  """) \
           .replace("&#8482;",   """  trademark  """) \
           .replace("&#8501;",   """  aleph  """) \
           .replace("&#alef;",   """  aleph  """) \
           .replace("&#8592;",   """  <-  """) \
           .replace("&#8593;",   """   uparrow """) \
           .replace("&#8594;",   """   ->  """) \
           .replace("&#8595;",   """  downarrow  """) \
           .replace("&#8596;",   """   <->  """) \
           .replace("&#8629;",   """  downarrow-corner-left  """) \
           .replace("&#8656;",   """ <--  """) \
           .replace("&#Unicode;",   """ <-  """) \
           .replace("&#8657;",   """  double-uparrow """) \
           .replace("&#8658;",   """ -->  """) \
           .replace("&#Unicode;",   """ ->  """) \
           .replace("&#8659;",   """ double-downarrow  """) \
           .replace("&#8660;",   """  <--> """) \
           .replace("&#8704;",   """  \forall  """) \
           .replace("&#8706;",   """   \partial  """) \
           .replace("&#8707;",   """ \exists  """) \
           .replace("&#8709;",   """  \emptyset  """) \
           .replace("&#8711;",   """  \nabla """) \
           .replace("&#8712;",   """  \in  """) \
           .replace("&#8713;",   """  \not\in   """) \
           .replace("&#8715;",   """  \ni  """) \
           .replace("&#8719;",   """  \prod  """) \
           .replace("&#prod;",   """  prod  """) \
           .replace("&#8721;",   """  \sum """) \
           .replace("&#sum;",   """  \sum""") \
           .replace("&#8722;",   """  -  """) \
           .replace("&#8727;",   """   *  """) \
           .replace("&#8730;",   """   \sqrt  """) \
           .replace("&#8733;",   """   \prop  """) \
           .replace("&#8734;",   """   \infinity  """) \
           .replace("&#8736;",   """  \angle  """) \
           .replace("&#8743;",   """  \wedge """) \
           .replace("&#8744;",   """  \vee """) \
           .replace("&#8745;",   """  \cap  """) \
           .replace("&#8746;",   """ \cup   """) \
           .replace("&#8747;",   """  \int  """) \
           .replace("&#8756;",   """ \therefore """) \
           .replace("&#8764;",   """   \sim """) \
           .replace("&#tilde;",   """~""") \
           .replace("&#8773;",   """ \approx """) \
           .replace("&#8776;",   """ \approx """) \
           .replace("&#8800;",   """ \neq """) \
           .replace("&#8801;",   """  =  """) \
           .replace("&#8804;",   """  \leq """) \
           .replace("&#8805;",   """  \geq  """) \
           .replace("&#8834;",   """   \subset """) \
           .replace("&#8835;",   """  \superset """) \
           .replace("&#note;",   """   \not\superset """) \
           .replace("&#8836;",   """  \not\subset """) \
           .replace("&#8838;",   """  \subseteq """) \
           .replace("&#8839;",   """  \superseteq """) \
           .replace("&#8853;",   """\oplus  """) \
           .replace("&#8855;",   """  \otimes """) \
           .replace("&#8869;",   """  \perp  """) \
           .replace("&#8901;",   """ \cdot  """) \
           .replace("&#dot;",   """ \cdot  """) \
           .replace("&#8968;",   """  \lceil """) \
           .replace("&#8969;",   """  \rceil  """) \
           .replace("&#8970;",   """ \lfloor  """) \
           .replace("&#8971;",   """  \rfloor """) \
           .replace("&#9001;",   """ \langle  """) \
           .replace("&#lang;",   """  \langle  """) \
           .replace("&#9002;",   """  \rangle  """) \
           .replace("&#rang;",   """ \rangle  """) \
           .replace("&#9674;",   """  \diamond """) \
           .replace("&#9824;",   """   \spades """) \
           .replace("&#black;",   """ \black  """) \
           .replace("&#9827;",   """  \clubs  """) \
           .replace("&#9829;",   """  \hearts  """) \
           .replace("&#9830;",   """  \diamonds """) \
           .replace("&lt;","<") \
           .replace( "&gt;",">") \
           .replace("&quot;","\"") \
           .replace("&#039;","'") \
           .replace("&mdash","--") \
           .replace("&nbsp;", " ") \
           .replace("&middot;","--") 

def encodeHTML(s=""):
    """encodeHTML(s) -> str

    Encode HTML special characters from their ASCII form to
    HTML entities.
    """

    return s.replace("&", "&amp;") \
            .replace("<", "&lt;") \
            .replace(">", "&gt;") \
            .replace("\"", "&quot;") \
            .replace("'", "&#039;") \
            .replace("--", "&mdash")



def canonicalize_sitename (url):
    urlparser = urllib2.urlparse.urlparse
    (scheme, netloc, path, params, query, fragment) = urlparser(url)
    ## Strip default server name
    netloc = simplify_netloc(netloc)
    return netloc

def simplify_netloc (netloc):
    if netloc.startswith('www.') or netloc.startswith('www-'):
        return netloc[4:]
    else:
        return netloc


if __name__ == '__main__':
    import os.path
    home = os.environ['HOME']
    os.chdir(os.path.join(home,'Desktop','SWIDSAS','software','scripts'))
    working_dir = os.path.join(home,'Desktop/SWIDSAS/tulasi_final/vaccine_only/topsy.com/vaccines11')
    # working_dir = os.path.join(home,'Desktop','SWIDSAS','data')
    filename2 = os.path.join(working_dir,\
                            'whooping+cough+outbreak.txt')
    #loc2 = convert_html_file(filename2)
    #loc2 = convert_bad_text_file(filename2)
    loc2 = crawler_package.clean_up_text_file.clean_and_chunk(filename2)
    
